/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Mozilla Communicator client code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Bill Law       law@netscape.com
 *   IBM Corp.
 */
#include <os2.h>

// Splash screen dialog ID.
#define IDD_SPLASH  100

// Splash screen bitmap ID.
#define IDB_SPLASH  101

ICON 1 FIREFOX_ICO

DLGTEMPLATE IDD_SPLASH DISCARDABLE 
BEGIN
    DIALOG "", IDD_SPLASH, 0, 0, 390, 261, , FCF_BORDER
    BEGIN
    END
END

//BITMAP IDB_SPLASH "splash.bmp"

#ifdef MOZ_STATIC_BUILD
#include "wdgtos2rc.h"

POINTER IDC_SELECTANCHOR   "..\\..\\widget\\src\\os2\\res\\\select.ptr"
POINTER IDC_ARROWNORTH     "..\\..\\widget\\src\\os2\\res\\arrow_north.ptr"
POINTER IDC_ARROWNORTHPLUS "..\\..\\widget\\src\\os2\\res\\arrow_north_plus.ptr"
POINTER IDC_ARROWSOUTH     "..\\..\\widget\\src\\os2\\res\\arrow_south.ptr"
POINTER IDC_ARROWSOUTHPLUS "..\\..\\widget\\src\\os2\\res\\arrow_south_plus.ptr"
POINTER IDC_ARROWWEST      "..\\..\\widget\\src\\os2\\res\\arrow_west.ptr"
POINTER IDC_ARROWWESTPLUS  "..\\..\\widget\\src\\os2\\res\\arrow_west_plus.ptr"
POINTER IDC_ARROWEAST      "..\\..\\widget\\src\\os2\\res\\arrow_east.ptr"
POINTER IDC_ARROWEASTPLUS  "..\\..\\widget\\src\\os2\\res\\arrow_east_plus.ptr"
POINTER IDC_GRAB           "..\\..\\widget\\src\\os2\\res\\grab.ptr"
POINTER IDC_GRABBING       "..\\..\\widget\\src\\os2\\res\\grabbing.ptr"
POINTER IDC_CELL           "..\\..\\widget\\src\\os2\\res\\cell.ptr"
POINTER IDC_COPY           "..\\..\\widget\\src\\os2\\res\\copy.ptr"
POINTER IDC_ALIAS          "..\\..\\widget\\src\\os2\\res\\aliasb.ptr"
POINTER IDC_ZOOMIN         "..\\..\\widget\\src\\os2\\res\\zoom_in.ptr"
POINTER IDC_ZOOMOUT        "..\\..\\widget\\src\\os2\\res\\zoom_out.ptr"

POINTER IDC_ARROWWAIT      "..\\..\\widget\\src\\os2\\res\\arrow_wait.ptr"
POINTER IDC_CROSS          "..\\..\\widget\\src\\os2\\res\\crosshair.ptr"
POINTER IDC_HELP           "..\\..\\widget\\src\\os2\\res\\help.ptr"

ICON    IDC_DNDURL         "..\\..\\widget\\src\\os2\\res\\dndurl.ico"
ICON    IDC_DNDTEXT        "..\\..\\widget\\src\\os2\\res\\dndtext.ico"
#endif
