/*
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *
 */

/* ===== checkbox.css ===================================================
  == Styles used by the XUL checkbox element.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: checkbox ::::: */

checkbox {
  -moz-appearance: checkbox-container;
  -moz-user-focus: ignore;
  -moz-box-align: center;
  margin: 2px 2px 4px 2px;
}

.checkbox-label-box {
  margin-left: 0px;
  padding: 0px;
}

.checkbox-icon {
  margin-right: 2px;
}

.checkbox-label {
  margin: 0 !important;
  padding-bottom: 1px;
}

/* ..... disabled state ..... */

checkbox[disabled="true"] {
  color: GrayText !important;
}

/* ::::: checkmark image ::::: */

.checkbox-check {
  -moz-appearance: checkbox;
  width: 14px;
  height: 12px;
}
  

