/*
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *
 */

/* ===== colorpicker.css ================================================
  == Styles used by the XUL colorpicker element.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: colorpicker button ::::: */
 
/* colorpicker button */

colorpicker[type="button"] {
  width: 38px;
  height: 24px;
  border: 1px solid #a7a7a7;
  background-color: ThreeDFace;
  padding: 3px;
  -moz-appearance: button-bevel;
}

.colorpicker-button-colorbox {
  border: 1px solid #000000;
}

/* ::::: colorpicker tiles ::::: */

.colorpickertile {
  width                 : 20px;
  height                : 20px;
  margin                : 1px;
}

.colorpickertile[selected="true"] {
  border : 1px outset #C0C0C0;

}

.colorpickertile[hover="true"] {
  border : 1px dotted #A7A7A7;
}

.cp-light[hover="true"] {
  border : 1px dotted #000000;
}
