/*
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *
 */

/* ===== groupbox.css ==================================================
  == Styles used by the XUL groupbox and related elements.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: groupbox ::::: */

groupbox {
  margin: 5px;
  border: 1px solid ThreeDShadow;
  -moz-border-radius: 5px;
  padding: 5px;
}

.groupbox-body {
  padding: inherit;
}

caption {
  margin: 0px 6px 2px 6px;
  padding: 0px 3px;
  background-color: -moz-Dialog;
}
