
@import url("chrome://global/skin/");
@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");


treechildren::-moz-tree-cell-text(user)
{
  font-weight: bold;
}

treechildren::-moz-tree-cell-text(locked)
{
  font-style: italic;
}
