LIBRARY         "mozical"

DESCRIPTION     '"""ical library"""'

EXETYPE		NT

SUBSYSTEM	WINDOWS

STUB		'WINSTUB.EXE'

VERSION		1.13

CODE		 EXECUTE READ

DATA		 READ WRITE

HEAPSIZE	1048576,4096

EXPORTS		
	icalcomponent_get_first_property	@1
	icalmemory_strdup			@2
	icalcomponent_free			@3
	icalcomponent_count_components		@4
	icalcomponent_add_component		@5
	icalcomponent_new_clone			@6
	icalcomponent_new			@7
	icalerrno				@8
	icalcomponent_get_span			@9
	icalerror_clear_errno			@10
	icalproperty_get_attendee		@11
	icalcomponent_get_next_property		@12
	icalcomponent_get_first_real_component	@13
	icalparameter_get_partstat		@14
	icalproperty_get_first_parameter	@15
	icalproperty_get_dtstamp		@16
	icalproperty_get_recurrenceid		@17
	icalproperty_get_uid			@18
	icalproperty_get_sequence		@19
	icalproperty_get_organizer		@20
	icalcomponent_isa			@21
	icalproperty_get_method			@22
	icalproperty_as_ical_string		@23
	icalerror_set_errno			@24
	icaltime_compare			@25
	icalreqstattype_from_string		@26
	icalparser_free				@27
	icalmemory_add_tmp_buffer		@28
	icalerror_stop_here			@29
	pvl_push				@30
	pvl_pop					@31
	pvl_newlist				@32
	pvl_free				@33
	pvl_data				@34
	pvl_next				@35
	icalcomponent_add_property		@36
	icalproperty_new_uid			@37
	icalvalue_get_datetime			@38
	icalcomponent_get_next_component	@39
	icalcomponent_get_first_component	@40
	icalcompiter_deref			@41
	icalcompiter_next			@42
	icalcomponent_begin_component		@43
	icalproperty_get_value			@44
	icalcomponent_vanew			@45
	icalproperty_vanew_uid			@46
	icalparameter_new_xliccomparetype	@47
	icalcomponent_is_valid			@48
	pvl_head				@49
	icalparser_parse			@50
	icalparser_set_gen_data			@51
	icalparser_new				@52
	icalcomponent_as_ical_string		@53
	icalcomponent_remove_component		@54
	icalvalue_as_ical_string		@55
	icalcomponent_get_current_component	@56
	input_buffer_p				@57
	input_buffer				@58
	icalvalue_compare			@59
	icalproperty_isa			@60
	icalparameter_get_xliccomparetype	@61
	icalvalue_free				@62
	icalvalue_new_from_string		@63
	icalproperty_kind_to_value_kind		@64
	icalproperty_kind_to_string		@65
	icalcomponent_kind_to_string		@66
	icalproperty_new_clone			@67
	icalproperty_new_prodid			@68
	icalproperty_new_version		@69
	icalproperty_new_comment		@70
	icalcomponent_set_dtstamp		@71
	icalproperty_new_dtstamp		@72
	icaltime_from_timet			@73
	icalproperty_set_parameter		@74
	icalparameter_new_partstat		@75
	icalcomponent_set_method		@76
	icalparameter_new_delegatedto		@77
	icalproperty_vanew_attendee		@78
	icalparameter_new_delegatedfrom		@79
	icalproperty_new_requeststatus		@80
	pvl_tail				@81
	pvl_insert_ordered			@82
	icaltime_is_null_time			@83
	icalcomponent_get_inner			@84
	icalproperty_new_method			@85
	icaltime_as_timet			@86
	icaltime_null_time			@87
	icalmemory_tmp_copy			@88
	icalparser_flex_input			@89
	icalparser_clear_flex_input		@90
	icalproperty_string_to_kind		@91
	icalcomponent_string_to_kind		@92
	icalproperty_set_summary		@93
	icalcomponent_new_vtodo			@94
	icalproperty_new_percentcomplete	@95
	icalproperty_new_due			@96
	icalproperty_new_completed		@97
	icalproperty_get_percentcomplete	@98
	icalproperty_get_due			@99
	icalproperty_get_completed		@100
	icalcomponent_new_vcalendar		@101
	icalcomponent_new_vevent		@102
	icalproperty_new_summary		@103
	icalproperty_new_description		@104
	icalproperty_new_location		@105
	icalproperty_new_categories		@106
	icalproperty_new_url			@107
	icalproperty_new_priority		@108
	icalproperty_new_status			@109
	icalproperty_new_class			@110
	icalproperty_new_x			@111
	icalproperty_set_x_name			@112
	icalcomponent_new_valarm		@113
	icalproperty_new_trigger		@114
	icalparameter_new_member		@115
	icalproperty_add_parameter		@116
	icaltime_as_ical_string			@117
	icalrecurrencetype_clear		@118
	icalproperty_new_rrule			@119
	icaltime_day_of_week			@120
	icalproperty_new_exdate			@121
	icalproperty_new_dtstart		@122
	icalproperty_new_dtend			@123
	icalproperty_new_contact		@124
	icalproperty_get_status			@125
	icalproperty_get_summary		@126
	icalproperty_get_description		@127
	icalproperty_get_location		@128
	icalproperty_get_categories		@129
	icaltime_adjust				@130
	icalrecur_iterator_free			@131
	icaltime_normalize			@132
	icalrecur_iterator_next			@133
	icalrecur_iterator_new			@134
	icalproperty_get_rrule			@135
	icalparser_parse_string			@136
	icalproperty_get_contact		@137
	icalproperty_get_exdate			@138
	icalrecurrencetype_day_position		@139
	icaltime_from_string			@140
	icalproperty_get_x_name			@141
	icalproperty_get_dtstart		@142
	icalproperty_get_value_as_string	@143
	icalparameter_get_member		@144
	icalproperty_get_class			@145
	icalproperty_get_priority		@146
	icalproperty_get_url			@147
	icaldurationtype_null_duration		@148
	icaltime_add				@149
	icaltime_subtract			@150
	icalproperty_get_dtend			@151
	icaldurationtype_is_null_duration	@152
	icalproperty_get_duration		@153
