/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *          Scott MacGregor <mscott@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


#include "nsISupports.idl"

/* nsIAlertsListener --> when a consumer asks the alerts service to put up an alert, they have the option
   of passing in an alert listener as well. The alert listener gets notified when the alert has gone away.
*/ 
[scriptable, uuid(A840623B-5B87-4e99-A74A-9EF0886C11DD)]
interface nsIAlertListener : nsISupports
{
  void onAlertFinished(in wstring aAlertCookie);
  void onAlertClickCallback(in wstring aAlertCookie); // if the user clicks on the link and you wanted notified....
};

[scriptable, uuid(7E8B76C6-A70C-4075-81AA-D48B6152E1B4)]
interface nsIAlertsService : nsISupports
{
   /* showAlertNotification: shows an animated alert icon 
      aImageUrl --> a url identifying an image to put in the alert.
	  aAlertTitle --> the title text for the alert (i.e. You have New Mail)
	  aAlertText --> the contents of the alert
	  alertTextClickable --> set to true if you wish to be notified if the user attempts to click
	                         the alert text. Also causes the alert text to look like a link.
	  aAlertCookie --> a blind cookie the alert will pass back to the consumer during the alert listener callbacks
   */
   void showAlertNotification(in string aImageUrl, in wstring aAlertTitle, 
                              in wstring aAlertText, in boolean alertTextClickable,
							  in wstring aAlertCookie,
							  in nsIAlertListener aAlertListener );
};

