/*  
  The contents of this file are subject to the Netscape Public
  License Version 1.1 (the "License"); you may not use this file
  except in compliance with the License. You may obtain a copy of
  the License at http://www.mozilla.org/NPL/
  
  Software distributed under the License is distributed on an "AS
  IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  implied. See the License for the specific language governing
  rights and limitations under the License.
  
  The Original Code is Mozilla Communicator client code, released
  March 31, 1998.
  
  The Initial Developer of the Original Code is Blake Ross. 
  Portions created by Blake Ross are
  Copyright (C) 2002 Blake Ross. All
  Rights Reserved.
  
  Contributor(s): 
    Blake Ross (blaker@netscape.com)
    Joe Hewitt (hewitt@netscape.com)
*/

window,
dialog {
  border: 1px solid ThreeDDarkShadow;
  padding: 6px;
}

#instructions {
  font-weight: bold;
  font-size: larger;
}